
;**************************************************************************
;*                          Arrow                                         *
;**************************************************************************
;**************************************************************************
;*                                                                        *
;*          Arrow / 30 LEDs with MAX6969                                  *
;*          v 1.06 - 16.02.2017                                           *
;*                                                                        *
;**************************************************************************
;*          Hardw.: PIC12F1822 used                                       *
;*          OSC.......: Internal osc. used     POWER.....:   ext. 5,0V    *
;**************************************************************************


;Vss (Pin 8) -- GND
;RA0 (Pin 7) -- MAX6969: Din
;RA1 (Pin 6) -- MAX6969: CLK
;RA2 (Pin 5) -- not used / reserved
;RA3 (Pin 4) -- not used
;RA4 (Pin 3) -- MAX6969: LE
;RA5 (Pin 2) -- MAX6969: -OE
;Vdd (Pin 1) -- +5V




 INCLUDE "P12F1822.INC"


_U001			     EQU     B'00100110000100'		;see Page 34 of Datasheet
_U002			     EQU     B'01100011111111'		;see Page 35 of Datasheet


	__CONFIG	_CONFIG1, _U001
	__CONFIG	_CONFIG2, _U002


;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"
TIMER3		EQU	H'22'		;delay routine

v_value		EQU	H'29'
v_MSB		EQU	H'2A'
v_LSB		EQU	H'2B'

v_m1		EQU	H'2C'
v_m2		EQU	H'2D'
v_s1		EQU	H'2E'
v_s2		EQU	H'2F'

v_tmp		EQU	H'30'
v_stop		EQU	H'31'
v_s_char	EQU	H'32'		;special character flag

v_demo2_n	EQU	H'33'
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------


;==========================================================================

		ORG	0		;Reset vector address
		goto	start

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************

start		nop
;
		clrf	INTCON		;disable interrupts

		movlw	B'00000000'
		movwf	OPTION_REG	;see datasheet Page 151

		movlw	H'03'
		movwf	BSR		;select BANK 3
		clrf	ANSELA

		movlw	H'01'
		movwf	BSR		;select BANK 1
		movlw	B'11000000'
		movwf	TRISA

		movf	OSCCON,0
		andlw	B'11101111'
		iorlw	B'01101000'
		movwf	OSCCON		;set 4MHz internal clock (default after reset is 500kHz)

		movlw	H'04'
		movwf	BSR		;select BANK 4
		clrf	WPUA

		clrf	BSR		;select BANK 0

		clrf	PORTA
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
main_loop	call	demo1
		call	demo1
		call	demo1
		call	demo1

		call	demo2

		goto	main_loop
;----------------------------------------------------------------------------
demo1		nop
		movlw	B'00000001'
		movwf	v_MSB
		movlw	B'11000111'
		movwf	v_LSB
		call	send_data
		bcf	PORTA,D'005'
		call	dr7

		movlw	B'00001111'
		movwf	v_MSB
		movlw	B'11010111'
		movwf	v_LSB
		call	send_data
		bcf	PORTA,D'005'
		call	dr7

		movlw	B'01111111'
		movwf	v_MSB
		movlw	B'11110111'
		movwf	v_LSB
		call	send_data
		bcf	PORTA,D'005'
		call	dr7

		movlw	H'00'
		movwf	v_MSB
		movlw	H'00'
		movwf	v_LSB
		call	send_data
		bcf	PORTA,D'005'
		call	dr7

		return
;---------------------------------------------------------------------------
demo2		movlw	D'010'
		movwf	v_demo2_n

demo2_loop	movlw	B'01111111'
		movwf	v_MSB
		movlw	B'11110111'
		movwf	v_LSB
		call	send_data
		bcf	PORTA,D'005'
		call	d1

		movlw	H'00'
		movwf	v_MSB
		movlw	H'00'
		movwf	v_LSB
		call	send_data
		bcf	PORTA,D'005'
		call	d1

		decfsz	v_demo2_n,1
		goto	demo2_loop

		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
send_data	nop

		bcf	PORTA,D'004'		;LE = 0

		movf	v_MSB,0
		movwf	v_value
		call	send_byte

		movf	v_LSB,0
		movwf	v_value
		call	send_byte

		bsf	PORTA,D'004'		;LOAD = 1
		nop
		bcf	PORTA,D'004'		;LOAD = 0
		return
;----------------------------------------------------------------------------
send_byte	nop
		bcf	PORTA,D'001'		;CLK = 0
;
		bcf	PORTA,D'000'		;DIN = 0
		btfsc	v_value,D'007'
		bsf	PORTA,D'000'		;DIN = 1
		call	send_bit

		bcf	PORTA,D'000'		;DIN = 0
		btfsc	v_value,D'006'
		bsf	PORTA,D'000'		;DIN = 1
		call	send_bit

		bcf	PORTA,D'000'		;DIN = 0
		btfsc	v_value,D'005'
		bsf	PORTA,D'000'		;DIN = 1
		call	send_bit

		bcf	PORTA,D'000'		;DIN = 0
		btfsc	v_value,D'004'
		bsf	PORTA,D'000'		;DIN = 1
		call	send_bit

		bcf	PORTA,D'000'		;DIN = 0
		btfsc	v_value,D'003'
		bsf	PORTA,D'000'		;DIN = 1
		call	send_bit

		bcf	PORTA,D'000'		;DIN = 0
		btfsc	v_value,D'002'
		bsf	PORTA,D'000'		;DIN = 1
		call	send_bit

		bcf	PORTA,D'000'		;DIN = 0
		btfsc	v_value,D'001'
		bsf	PORTA,D'000'		;DIN = 1
		call	send_bit

		bcf	PORTA,D'000'		;DIN = 0
		btfsc	v_value,D'000'
		bsf	PORTA,D'000'		;DIN = 1
		call	send_bit

		return
;----------------------------------------------------------------------------
send_bit	bsf	PORTA,D'001'		;CLK = 1
		nop
		bcf	PORTA,D'001'		;CLK = 0
		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------



;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;		*********************************************
;		*               delay routine               *
;		*********************************************

;------------------------------------

d2		movlw	D'255'
		movwf	TIMER2
DEL_LOOP1	movlw	D'255'
		movwf	TIMER1

DEL_LOOP2	decfsz	TIMER1,F
		goto	DEL_LOOP2
		decfsz	TIMER2,F
		goto	DEL_LOOP1
		return
;------------------------------------
dr7		call	d2
		return
;------------------------------------
dr2		call	d2
		call	d2
		return
;------------------------------------
dr10		call	d2
		call	d2
		call	d2
		call	d2
		call	d2
		call	d2
		call	d2
		call	d2
		call	d2
		call	d2
		return
;------------------------------------
d1		movlw	D'100'
		movwf	TIMER2
DEL_LOOP11	movlw	D'255'
		movwf	TIMER1

DEL_LOOP12	decfsz	TIMER1,F
		goto	DEL_LOOP12
		decfsz	TIMER2,F
		goto	DEL_LOOP11
		return
;------------------------------------

d55		movlw	D'255'
		movwf	TIMER2
d55_loop	decfsz	TIMER2,F
		goto	d55_loop
		return

;------------------------------------
;------------------------------------
;tuning:
;	005 x 255 x 255 = 325.125 -- a little bit too fast
;	006 x 218 x 250 = 327.000 -- still too fast
;	006 x 219 x 250 = 328.500 -- still too fast
;	006 x 221 x 250 = 331.500 -- very little too slow; but for the application ok...
;------------------------------------
d_1sec		movlw	D'006'
		movwf	TIMER3
DEL_LOOPx0	movlw	D'221'
		movwf	TIMER2
DEL_LOOPx1	movlw	D'250'
		movwf	TIMER1
DEL_LOOPx2	decfsz	TIMER1,F
		goto	DEL_LOOPx2
		decfsz	TIMER2,F
		goto	DEL_LOOPx1
		decfsz	TIMER3,F
		goto	DEL_LOOPx0
		return
;------------------------------------
;------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

		END